@@ -11,6 +11,8 @@ use indexmap::IndexMap;
 use serde_json as json;
 use tokio::time;
 
+use crate::statehub::events::WithEvent;
+
 use super::*;
 
 mod gitlab;
@@ -248,7 +250,12 @@ impl StateHub {
             locations.show(),
         ))?;
 
-        kubectl.install_statehub_cluster(namespace).await?;
+        let event = self.install_statehub_cluster_event(namespace);
+        let event = kubectl
+            .install_statehub_cluster(namespace)
+            .await
+            .with_event(event);
+        self.report_event(event).await?;
 
         self.setup_cluster_token_helper(&kubectl, &cluster, register_cluster)
             .await?;
@@ -291,13 +298,17 @@ impl StateHub {
         {
             anyhow::ensure!(
                 *name == statehub_cluster_name,
-                "Cluster name mismatch: K8s thinks it is '{}', while Statehub thinks it is  '{}'",
+                "Cluster name mismatch: K8s thinks it is '{}', while Statehub thinks it is '{}'",
                 statehub_cluster_name,
                 name,
             );
 
             self.helm_uninstall_helper(&kubectl, &cluster, &namespace, &helm)
                 .await?;
+
+            let event = self.uninstall_statehub_cluster_event(&namespace);
+            let event = kubectl.uninstall_statehub_cluster().await.with_event(event);
+            self.report_event(event).await?;
         }
 
         self.relinquish_states_helper(name).await?;
@@ -307,7 +318,6 @@ impl StateHub {
             .await
             .print(&self.stdout, &self.style, self.json)?;
 
-        kubectl.uninstall_statehub_cluster().await?;
         Ok(())
     }
 
@@ -495,7 +505,8 @@ or run following commands before running 'statehub register-cluster`
         let default_state = register_cluster.default_state.as_deref().unwrap_or("");
         let api = self.api.url("");
 
-        kubectl
+        let event = self.setup_configmap_event(&register_cluster.namespace);
+        let event = kubectl
             .store_configmap(
                 &register_cluster.namespace,
                 &cluster.name,
@@ -503,7 +514,9 @@ or run following commands before running 'statehub register-cluster`
                 &api,
                 &register_cluster.cleanup_grace,
             )
-            .await?;
+            .await
+            .with_event(event);
+        self.report_event(event).await?;
 
         Ok(())
     }
@@ -516,9 +529,14 @@ or run following commands before running 'statehub register-cluster`
     ) -> anyhow::Result<()> {
         let token = self.api.issue_cluster_token(&cluster.name).await?;
         self.verbosely(format!("Issued token {} for {}", token.token, cluster))?;
-        kubectl
+
+        let event = self.store_cluster_token_event(&register_cluster.namespace);
+        let event = kubectl
             .store_cluster_token(&register_cluster.namespace, &token.token)
-            .await?;
+            .await
+            .with_event(event);
+        self.report_event(event).await?;
+
         Ok(())
     }
 
@@ -621,7 +639,9 @@ or run following commands before running 'statehub register-cluster`
         let kube_context = kubectl.context();
         let commands = helm.install_command(cluster, Some(kube_context), namespace);
 
-        let (stdout, stderr) = helm.execute(commands).await?;
+        let event = self.helm_install_event(namespace, &commands);
+        let event = helm.execute(commands).await.with_event(event);
+        let (stdout, stderr) = self.report_event(event).await?;
 
         self.verbosely(stdout)?;
         self.error(stderr)?;
